<?php


namespace Orbipay\PaymentsApi\Client;


use Exception;

use lf4php\LoggerFactory;

class OrbipayApiException extends Exception
{

    /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\ErrorResponse
     */
    private $error_response;

    private $http_status_code;
    
    /**
     * @return \Orbipay\PaymentsApi\Client\Wrappers\ErrorResponse
     */
    public function getErrorResponse()
    {
        return $this->error_response;
    }

    public function getHttpStatusCode()
    {
        return $this->http_status_code;
    }
    
     /**
     * @var \Orbipay\PaymentsApi\Client\Wrappers\InvocationContext
     */

    private $invocation_context;

    /**
     * @return \Orbipay\PaymentsApi\Client\Wrappers\InvocationContext
     */
    public function getInvocationContext()
    {
        return $this->invocation_context;
    }
 	
 	protected $isTimedOut;

    /**
     * @return bool
     */
    public function isTimedOut()
    {
        return $this->isTimedOut;
    }
    
    /**
     * OrbipayApiException constructor.
     * @param string $message
     * @param int $code
     * @param mixed $previous
     */
    public function __construct($message = null, $code = 0, $previous = null)
    {
        parent::__construct($message, $code, $previous);
    }

    public static function constructFromApiException(ApiException $apiException = null)
    {
    
	    $logger = LoggerFactory::getLogger(__CLASS__);
        if($logger->isDebugEnabled()){
            $logger->debug('HTTP Status Code  :'.$apiException->getCode());
            $logger->debug('Response Headers  :'.json_encode($apiException->getResponseHeaders()));
            $logger->debug('HTTP Response Body:'.CommonUtil::get_masked_string(json_decode($apiException->getResponseBody())));
        }
        
        $exception = new OrbipayApiException($apiException->getMessage(), $apiException->getCode(), $apiException);

        $exception->http_status_code = $apiException->getCode();
     	$exception->invocation_context = CommonUtil::getInvocationContext($apiException->getResponseHeaders());

        if (is_string($apiException->getResponseBody()) && CommonUtil::isJson($apiException->getResponseBody())) {
            $exception->error_response = ObjectSerializer::deserialize(json_decode($apiException->getResponseBody()),'\Orbipay\PaymentsApi\Client\Wrappers\ErrorResponse',null);
        }
        $exception->isTimedOut = $apiException->isTimedOut();
        
        return $exception;
    }
}